/*
  PWM pulses generation for AC/DC boost circuit - 2022
*/

// setting hardware pins
const byte ledPin = 13; //onboard led for triggering control
const byte pwm1 = 9;  // pwm output 1
const byte pwm2 = 11; // pwm output 2
const byte interruptPin = 3;// interrupt input pin for triggering

// setting control parameters
int Td = 1; // delay time (in percent range: 1-100, >0)
int D = 75; // duty cycle (in percent range: 51-100, >50%)

// working variables
volatile byte state = LOW;
volatile byte flag1 = HIGH;
volatile byte flag2 = HIGH;
volatile byte flag3 = HIGH;
volatile byte flag4 = HIGH;
volatile byte flag_p = HIGH;
volatile byte flag_n = HIGH;

volatile unsigned long pp = 20000; // period inizialization
volatile unsigned long c_p = 0; // current time in trigger - positive pulse
volatile unsigned long p_p = 0; // previous time in trigger - positive pulse
volatile unsigned long c_n = 0; // current time in trigger - negative pulse
volatile unsigned long p_n = 0; // previous time in trigger - negative pulse
volatile unsigned long c_loop = 0; // current time in loop cycle
volatile unsigned long p_loopp = 0; // previous time in loop - positive pulse
volatile unsigned long p_loopn = 0; // previous time in loop - negative pulse

long dT, DT; // delay time and duty cycle in us

//-------------------------------------- SETUP -----------------------------
void setup() {
  Serial.begin(9600); // initialize serial connection
  pinMode(ledPin, OUTPUT); // sets pin mode
  pinMode(pwm1, OUTPUT); // sets pin mode
  pinMode(pwm2, OUTPUT); // sets pin mode
  digitalWrite(pwm1, LOW); // inizialize pwm output
  digitalWrite(pwm2, LOW); // inizialize pwm output
  //  pinMode(interruptPin, INPUT_PULLUP);
  // every change 0<->1 at interruptPin calls function "meas"
  attachInterrupt(digitalPinToInterrupt(interruptPin), meas, CHANGE);

  // parameter input through serial monitor
  //Menu();

  //iniziamo
  Serial.println("Start! ");
  p_loopp = micros(); // store current time in us
  p_loopn = p_loopp;  // store current time in us
}//------------------------------------ END SETUP -------------------------
//-------------------------------------- LOOP -----------------------------
void loop() {
  c_loop = micros(); // store current time in us
  if (flag_p == HIGH)   timing_p(); // a positive pulse is occurring ("meas" set the flag_p)
  if (flag_n == HIGH)   timing_n(); // a negative pulse is occurring ("meas" set the flag_n)

  if ((c_loop - p_loopp >= dT) && (flag1 == HIGH)) {
    digitalWrite(pwm2, HIGH); // sets
    flag1 = LOW;
  }
  if ((c_loop - p_loopp >= dT + DT) && (flag2 == HIGH)) {
    digitalWrite(pwm2, LOW);  // sets
    flag2 = LOW;
  }
  if ((c_loop - p_loopn >= dT) && (flag3 == HIGH)) {
    digitalWrite(pwm1, HIGH);  // sets
    flag3 = LOW;
  }
  if ((c_loop - p_loopn >= dT + DT) && (flag4 == HIGH)) {
    digitalWrite(pwm1, LOW);  // sets
    flag4 = LOW;
  }
  //  digitalWrite(ledPin, state);  // sets the LED on-off at each trigger
} //------------------------------------ END LOOP ---------------------------------

void meas() { // at each trigger -- period update
    p_p = c_p; // store previous trigger timing
    c_p = micros(); // update actual trigger timing
    pp = 2*(c_p - p_p); // period calculation
    dT = Td * (pp / 100); // delay time in us
    DT = D * (pp / 100); //  duty cycle in us
  if (digitalRead(interruptPin) == HIGH) {// positive pulse
    flag_p = HIGH; // we are in the positive pulse
  }
  else { // negative pulse
    flag_n = HIGH; // we are in the negative pulse
  }
}//------------------------------------- END MEAS ---------------------------------

void timing_p() { // flag reset for positive pulse
  p_loopp = c_loop;  // sets positive previous time
  flag1 = HIGH;
  flag2 = HIGH;
  flag_p = LOW;
}
void timing_n() { // flag reset for negative pulse
  p_loopn = c_loop;  // sets negative previous time
  flag3 = HIGH;
  flag4 = HIGH;
  flag_n = LOW;
}


void Menu() { // function for external parameters input through serial terminal
  Banner();
  for (;;) {
    switch (Serial.read()) {
      //--------------------------------------
      case '1': {
          Serial.println("Insert delay time Td in percent (1:100): ");
          while (!Serial.available()) { } //Do Nothing unless there is a reply!
          Td = Serial.parseInt();
          Banner();
          break;
        }
      //--------------------------------------
      case '2': {
          Serial.println("Insert Duty cycle D in percent (1:99): ");
          while (!Serial.available()) { } //Do Nothing unless there is a reply!
          D = Serial.parseInt();
          Banner();
          break;
        }
      //--------------------------------------
      case '3': return;
      default: continue;  // includes the case 'no input'
    }
  }
}//------------------------------------- END MENU ---------------------------------

void Banner() { // Banner to show the actual parameters
  Serial.println("--------------------------------------------------");
  Serial.println("2 PWM generation for AC/DC boost  (July 2022)");
  Serial.println(' ');
  Serial.print("1) Delay time, Td = "); Serial.println(Td); // actual value of Td
  Serial.print("2) Duty cycle, D = "); Serial.println(D);   // actual value of D
  Serial.println("3) Start "); // start the routine
}//------------------------------------- END BANNER -------------------------------
